; Calculate Cube Checksum v1 by Guillermito
; Fix the checksum after firmware modification 
; Only needed for DAH-1500 cube, not DAH-1500i
;
; Imput    : fw.bin
; Output   : fw_checksum_fixed.bin
;
; Assemble : TASM32 /ml /m3 /z /t calculate_cube_checksum
; Link     : TLINK32 -Tpe -aa calculate_cube_checksum,,,import32


callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "Mobiblu firmware file",0,"*.bin",0
                  db "All files",0,"*.*",0,0
namebuffer            db 255 dup(0)
result_title      db "Calculate Cube Checksum by Guillermito", 0
bad_size          db "File is not exactly 512 Kb",0
no_sig            db "No 55AA",0
all_ok            db "Done !",0

file_name          db "fw_checksum_fixed.bin",0
file_handle2       dd ?
password_or_not    dd ?
file_size          dd ?
file_handle        dd ?
file_mem_buffer    dd ?
file_nb_bytes_read dd ?
results            dd ?
pixels_data_start  dd ?
size_hidden_data   dd ?

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;---------- check if size is correct --------------

cmp eax, 524288d
je no_trouble

;-------- bad size -----------

push 0
push offset result_title
push offset bad_size
push 0
callW MessageBoxA
jmp close_file

;--------- alloc memory for file --------

no_trouble:
mov eax, file_size
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax
add eax, file_size
add eax, 1024
mov results, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;-------- find firmware signature ---------

mov esi, file_mem_buffer
mov ax, word ptr [esi] 
cmp ax, 0AA55h
je no_trouble2

;-------- no sig -----------

push 0
push offset result_title
push offset no_sig
push 0
callW MessageBoxA
jmp free_memory

;------- calculate ------

no_trouble2:
mov ecx, file_size
shr ecx, 1
mov bx, 0ffffh
xor edx, edx
loopaga:
lodsb
 cmp edx, 46h/2
 jne addaga
 xor eax, eax
 addaga:
add bl, al
rol bx, 8
lodsb
 cmp edx, 46h/2
 jne addaga2
 xor eax, eax
 addaga2:
add bl, al
ror bx, 8
inc edx
cmp edx, ecx
jne loopaga

;---------- replace old checksum by new --------------

mov edi, file_mem_buffer
add edi, 46h
xchg eax, ebx
stosw

;------- open a new file on the disk ----------------

push 0                  ;handle of file with attributes to copy (0=Ludwig)
push 80h                ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                  ;how to create (2=OPEN_ALWAYS)
push 0                  ;address of security descriptor (0=Ludwig)
push 0                  ;share mode (0=Prevents the file from being shared)
push 40000000h          ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name   ;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read          ;address of number of bytes written 
push file_size                          ;number of bytes to write
push file_mem_buffer                    ;address of data to write to file
push file_handle2                       ;handle of file to write to
callW WriteFile

;--------- victory message -------------

push 0
push offset result_title
push offset all_ok
push 0
callW MessageBoxA

;------- close it ----------------

push file_handle2
callW CloseHandle

;------- close memory and file ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;----------- exit ----------

close_file:
push dword ptr file_handle
callW CloseHandle

end:
push -1
callW ExitProcess

end programme
